<?php

namespace Drupal\eca_remote_request\Plugin\Action;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TypedData\Plugin\DataType\StringData;
use Drupal\eca\Plugin\Action\ConfigurableActionBase;
use Drupal\eca\Plugin\DataType\DataTransferObject;

/**
 * Describes the eca_remote_request convert_json_to_list action.
 *
 * @Action(
 *   id = "eca_remote_request_convert_json_to_list",
 *   label = @Translation("Convert JSON To List"),
 *   description = @Translation("Convert JSON string to a list to use data within a workflow."),
 *   eca_version_introduced = "2.0.0"
 * )
 */
class ConvertJsonToListAction extends ConfigurableActionBase {

  /**
   * {@inheritdoc}
   */
  public function access($object, ?AccountInterface $account = NULL, $return_as_object = FALSE): bool|AccessResultInterface {
    $access_result = AccessResult::allowed();
    return $return_as_object ? $access_result : $access_result->isAllowed();
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $tokenService = $this->tokenService;
    $config = &$this->configuration;

    $dataString = $tokenService->getOrReplace($config['data']);

    if (!($dataString instanceof StringData)) {
      return;
    }

    $dataArray = Json::decode($dataString->getValue());
    if (json_last_error() !== JSON_ERROR_NONE || !is_array($dataArray)) {
      return;
    }
    $data = DataTransferObject::create($dataArray);

    $tokenName = isset($config['token_name']) ? trim($config['token_name']) : '';
    if ($tokenName !== '') {
      $tokenService->addTokenData($tokenName, $data);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'data' => '',
      'token_name' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['data'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Data string'),
      '#description' => $this->t('Data to convert to a list. The following can reference a token or JSON data directly.'),
      '#default_value' => $this->configuration['data'],
      '#required' => TRUE,
    ];
    $form['token_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Token name holding the response'),
      '#default_value' => $this->configuration['token_name'],
      '#description' => $this->t('The name of the token to hold the selection.'),
      '#eca_token_reference' => TRUE,
    ];
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['data'] = $form_state->getValue('data');
    $this->configuration['token_name'] = $form_state->getValue('token_name');
    parent::submitConfigurationForm($form, $form_state);
  }

}
