<?php

namespace Drupal\eca_remote_request\Plugin\Action;

use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\eca\Plugin\Action\ConfigurableActionBase;
use Drupal\eca\Plugin\DataType\DataTransferObject;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Describes the eca_remote_request run_remote_requests action.
 *
 * @Action(
 *   id = "eca_remote_request_run_remote_requests",
 *   label = @Translation("Run Remote Requests"),
 *   description = @Translation("Run remote requests to use within ECA workflows."),
 *   eca_version_introduced = "2.0.0"
 * )
 */
class RunRemoteRequestsAction extends ConfigurableActionBase {

  /**
   * {@inheritdoc}
   */
  public function access($object, ?AccountInterface $account = NULL, $return_as_object = FALSE): bool|AccessResultInterface {
    $access_result = AccessResult::allowed();
    return $return_as_object ? $access_result : $access_result->isAllowed();
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $tokenService = $this->tokenService;
    $config = &$this->configuration;

    $client = new Client();
    $options = [];
    try {
      $optionData = $tokenService->getOrReplace($config['options']);
      $options = Yaml::decode($optionData);
      $options = is_array($options) ? $options : [];
    }
    catch (\Exception $exception) {
      $this->logger->error($exception->getMessage());
    }

    if (in_array($config['method'], ['PATCH', 'POST', 'PUT'])) {
      $key = $config['post_type'];

      try {
        $sendData = $this->tokenService->getOrReplace($config['data']);
        $options[$key] = Yaml::decode($sendData);
      }
      catch (\Exception $exception) {
        $this->logger->error($exception->getMessage());
      }
    }

    try {
      $response = $client->request($config['method'], $config['url'], $options);
    }
    catch (GuzzleException $guzzleException) {
      $this->logger->error($guzzleException->getMessage());
      $response = $guzzleException->getResponse();
    }

    $data = DataTransferObject::create([
      'status' => $response->getStatusCode(),
      'headers' => $response->getHeaders(),
      'contents' => $response->getBody()->getContents(),
    ]);

    $tokenName = isset($config['token_name']) ? trim($config['token_name']) : '';
    if ($tokenName !== '') {
      $tokenService->addTokenData($tokenName, $data);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'url' => '',
      'method' => 'GET',
      'post_type' => 'form_params',
      'data' => '',
      'options' => '',
      'token_name' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['url'],
      '#description' => $this->t('Endpoint to send data to.'),
    ];
    $form['method'] = [
      '#type' => 'select',
      '#title' => $this->t('Method'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['method'],
      '#options' => [
        'GET' => $this->t('GET'),
        'POST' => $this->t('POST'),
        'DELETE' => $this->t('DELETE'),
        'PATCH' => $this->t('PATCH'),
        'PUT' => $this->t('PUT'),
        'HEAD' => $this->t('HEAD'),
        'OPTIONS' => $this->t('OPTIONS'),
      ],
    ];
    $form['post_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Post type'),
      '#default_value' => $this->configuration['post_type'],
      '#description' => $this->t('Enter type of data that should be sent over in all remote POST/PUT/PATCH requests.'),
      '#options' => [
        'form_params' => $this->t('Form Data'),
        'json' => $this->t('JSON Data'),
      ],
    ];
    $form['data'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Data'),
      '#default_value' => $this->configuration['data'],
      '#description' => $this->t('Enter custom data that will be included in all remote POST/PUT/PATCH requests.'),
      '#rows' => 5,
    ];
    $form['options'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom Options'),
      '#default_value' => $this->configuration['options'],
      '#description' => $this->t('Enter custom <a href="https://docs.guzzlephp.org/en/stable/request-options.html" target="_blank">request options</a> that will be used by the Guzzle HTTP client. Request options can include custom headers.'),
      '#rows' => 5,
    ];
    $form['token_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Token name holding the response'),
      '#default_value' => $this->configuration['token_name'],
      '#description' => $this->t('The name of the token to hold the selection.'),
      '#eca_token_reference' => TRUE,
    ];
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    try {
      Yaml::decode($form_state->getValue('data'));
    }
    catch (\Exception $exception) {
      $this->logger->error($exception->getMessage());
      $form_state->setErrorByName('data', 'Yaml data is not properly formatted.');
    }

    try {
      Yaml::decode($form_state->getValue('options'));
    }
    catch (\Exception $exception) {
      $this->logger->error($exception->getMessage());
      $form_state->setErrorByName('options', 'Yaml data is not properly formatted.');
    }

    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['url'] = $form_state->getValue('url');
    $this->configuration['method'] = $form_state->getValue('method');
    $this->configuration['post_type'] = $form_state->getValue('post_type');
    $this->configuration['data'] = $form_state->getValue('data');
    $this->configuration['options'] = $form_state->getValue('options');
    $this->configuration['token_name'] = $form_state->getValue('token_name');
    parent::submitConfigurationForm($form, $form_state);
  }

}
